#include "FrSkySerial.h"
#include "FrSkySPortGPS.h"

void processSPortGPSPacket(uint8_t *packet, gpsdata_ *gps)
{
  uint16_t appId = SPORT_APPID_U16(packet);
  if (appId >= GPS_SPEED_FIRST_ID && appId <= GPS_SPEED_LAST_ID) 
  {
    gps->gpsSpeed_bp = (uint16_t) (SPORT_DATA_U32(packet)/1000);
    if (gps->gpsSpeed_bp > gps->maxGpsSpeed)
      gps->maxGpsSpeed = gps->gpsSpeed_bp;
  }
  else if (appId >= GPS_TIME_DATE_FIRST_ID && appId <= GPS_TIME_DATE_LAST_ID) 
  {
    uint32_t gps_time_date = SPORT_DATA_U32(packet);
    if (gps_time_date & 0xFF) {
      gps->year = (gps_time_date >> 24)&0xFF;
      gps->month = (gps_time_date >> 16)&0xFF;
      gps->day = (gps_time_date >> 8)&0xFF;
    }
    else {
      gps->hour = (gps_time_date >> 24)&0xFF;
      gps->min = (gps_time_date >> 16)&0xFF;
      gps->sec = (gps_time_date >> 8)&0xFF;
      gps->hour = ((uint8_t) (gps->hour + TIMEZONE + 24)) % 24;
    }
  }
  else if (appId >= GPS_COURS_FIRST_ID && appId <= GPS_COURS_LAST_ID)
  {
    uint32_t course = SPORT_DATA_U32(packet);
    gps->gpsCourse_bp = course / 100;
    gps->gpsCourse_ap = course % 100;
  }
  else if (appId >= GPS_ALT_FIRST_ID && appId <= GPS_ALT_LAST_ID) {
    gps->gpsAltitude = SPORT_DATA_S32(packet);

    if (gps->gpsAltitude > gps->maxAltitude)
      gps->maxAltitude = gps->gpsAltitude;
    if (gps->gpsAltitude < gps->minAltitude)
      gps->minAltitude = gps->gpsAltitude;
  }
  else if (appId >= GPS_LONG_LATI_FIRST_ID && appId <= GPS_LONG_LATI_LAST_ID) {
    uint32_t gps_long_lati_data = SPORT_DATA_U32(packet);
    uint32_t gps_long_lati_b1w, gps_long_lati_a1w;
    gps_long_lati_b1w = (gps_long_lati_data & 0x3fffffff) / 10000;
    gps_long_lati_a1w = (gps_long_lati_data & 0x3fffffff) % 10000;
    switch ((gps_long_lati_data >> 30) & 0x3) {
      case 0:
        gps->gpsLatitude_bp = (gps_long_lati_b1w / 60 * 100) + (gps_long_lati_b1w % 60);
        gps->gpsLatitude_ap = gps_long_lati_a1w;
        gps->gpsLatitudeNS = 'N';
        break;
      case 1:
        gps->gpsLatitude_bp = (gps_long_lati_b1w / 60 * 100) + (gps_long_lati_b1w % 60);
        gps->gpsLatitude_ap = gps_long_lati_a1w;
        gps->gpsLatitudeNS = 'S';
        break;
      case 2:
        gps->gpsLongitude_bp = (gps_long_lati_b1w / 60 * 100) + (gps_long_lati_b1w % 60);
        gps->gpsLongitude_ap = gps_long_lati_a1w;
        gps->gpsLongitudeEW = 'E';
        break;
      case 3:
        gps->gpsLongitude_bp = (gps_long_lati_b1w / 60 * 100) + (gps_long_lati_b1w % 60);
        gps->gpsLongitude_ap = gps_long_lati_a1w;
        gps->gpsLongitudeEW = 'W';
        break;
    }
    if (gps->gpsLongitudeEW && gps->gpsLatitudeNS) {
      gps->gpsFix = 1;
    }
    else if (gps->gpsFix > 0) {
      gps->gpsFix = 0;
    }
  }
}
