#include "mk20dx128.h"

#include "usbtsy/usb_dev.h"
#include "usbtsy/usb_debug_hid.h"

#include "frsky/FrSkySerial.h"
#include "frsky/FrSkySPortGPS.h"

#define _BV(n) (1<<(n))

int main(void) {
	PORTC_PCR5  |= _BV(8); // Digital IO
	GPIOC_PDDR |= _BV(5);
	GPIOC_PDOR |= _BV(5);
	
	uint8_t hidBuff[64]={0};

	usb_init();
	while(!usb_configuration);
  
  FrSkySerial_init(57600);
  gpsdata_ gps = {0}; gps.gpsFix = -1;
  while(1) 
  {
    uint16_t n = FrSkySerial_available();
    if(n < FRSKY_SPORT_PACKET_SIZE)
    {
      FrSkySerial_TxMode();
      FrSkySerial_putchar(0x7E); 
      FrSkySerial_putchar(0x83); 
      FrSkySerial_flush();
      FrSkySerial_RxMode();
      uint32_t d=0; while(d++<0xFFFFF); // Necessary delay...
    } 
    else 
    {
      GPIOC_PTOR = _BV(5);
      if((*hidBuff = FrSkySerial_getchar()) == 0x10)
      {
        FrSkySerial_read(FRSKY_SPORT_PACKET_SIZE-1,hidBuff+1);
        if(checkSportPacket(hidBuff)) {
          processSPortGPSPacket(hidBuff,&gps);
        }
        if(gps.gpsFix > 0)
        {
          usb_debug_print("\nFix !");
          usb_debug_print("\n  Long: "); usb_debug_phex16(gps.gpsLongitude_bp); 
                 usb_debug_putchar('.'); usb_debug_phex16(gps.gpsLongitude_ap); 
                  usb_debug_print(": "); usb_debug_putchar(gps.gpsLongitudeEW); 
          usb_debug_print("\n   Lat: "); usb_debug_phex16(gps.gpsLatitude_bp); 
                 usb_debug_putchar('.'); usb_debug_phex16(gps.gpsLatitude_ap); 
                  usb_debug_print(": "); usb_debug_putchar(gps.gpsLatitudeNS);
          usb_debug_print("\n   Alt: "); usb_debug_phex32(gps.gpsAltitude);
          usb_debug_print("\n   Spd: "); usb_debug_phex16(gps.gpsSpeed_bp);
                 usb_debug_putchar('.'); usb_debug_phex16(gps.gpsSpeed_ap);
          usb_debug_print("\n  Cour: "); usb_debug_phex16(gps.gpsCourse_bp); 
                 usb_debug_putchar('.'); usb_debug_phex16(gps.gpsCourse_ap); 
          usb_debug_print("\n  Time:"); 
          usb_debug_print("\n   YMD: "); usb_debug_phex(gps.year); 
                 usb_debug_putchar('/'); usb_debug_phex(gps.month); 
                 usb_debug_putchar('/'); usb_debug_phex(gps.day);
          usb_debug_print("\n   HMS: "); usb_debug_phex(gps.hour); 
                 usb_debug_putchar(':'); usb_debug_phex(gps.min); 
                 usb_debug_putchar(':'); usb_debug_phex16(gps.sec);
        }
        else if(gps.gpsFix < 0)
        {
          usb_debug_putchar('.');
        }
        else
        {
          usb_debug_putchar('*');
        } 
        usb_debug_flush_output();
      } 
    }		
	}
}



